package MT::Plugin::SKR::GetTweet;

use strict;
use MT 4;
use MT::Util;
use LWP::Simple;
use XML::Simple;
use HTTP::Date;
use Encode;

use vars qw( $MYNAME $VERSION );
$MYNAME = 'GetTweet';
$VERSION = '0.21';

use constant REGET_SPAN_TIME => 180;

use base qw( MT::Plugin );
my $plugin = __PACKAGE__->new ({
        name => $MYNAME,
        id => lc $MYNAME,
        key => lc $MYNAME,
        version => $VERSION,
        author_name => 'SKYARC System Co.,Ltd.',
        author_link => 'http://www.skyarc.co.jp/',
        doc_link => 'http://www.skyarc.co.jp/engineerblog/entry/gettweet.html',
        description => <<HTMLHEREDOC,
<__trans phrase="Get your tweet and embed them in your site.">
HTMLHEREDOC
        config_template => \&_config_template,
        settings => new MT::PluginSettings ([
            [ 'feed_url', { Default => undef } ],
            [ 'last_get', { Default => 0 } ],
            [ 'last_feed', { Default => undef } ],
        ]),
});
MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        tags => {
            block => {
                $MYNAME => \&_hdlr_get_tweet,
            },
            function => {
                TweetBody => \&_hdlr_tweet_body,
                TweetLink => \&_hdlr_tweet_link,
                TweetDate => \&_hdlr_tweet_date,
            },
        },
    });
}



###
sub _config_template {
    return <<'HTMLHEREDOC';
<mtapp:setting
    id="feed_url"
    label="<__trans phrase="Feed URL">"
    hint="">
<input type="text" name="feed_url" value="<TMPL_VAR NAME=FEED_URL ESCAPE=HTML>" />
</mtapp:setting>
HTMLHEREDOC
}



###
sub _hdlr_get_tweet {
    my ($ctx, $args, $cond) = @_;

    my $blog = $ctx->stash ('blog')
        or return ''; # never reach here

    my $last_get = $plugin->get_config_value ('last_get', 'blog:'. $blog->id)
        || 0;

    my $rss;
    if (REGET_SPAN_TIME < time - $last_get) {
        # Get tweets with network
        my $feed_url = $plugin->get_config_value ('feed_url', 'blog:'. $blog->id)
            || $plugin->get_config_value ('feed_url', 'system')
                or return ''; # not yet configured
        $rss = LWP::Simple::get ($feed_url)
            or return "Failed to retrieve $feed_url";
        # Store in PluginData
        $plugin->save_config ({
            feed_url => $feed_url, last_get => time, last_feed => $rss,
        }, 'blog:'. $blog->id);
    }
    else {
        # Use cached one
        $rss = $plugin->get_config_value ('last_feed', 'blog:'. $blog->id)
            || '';
    }

    my $parser = XML::Simple->new;
    my $data = eval { $parser->XMLin($rss, ForceArray => [qw/item/]) };
        return "Failed to parse XML feed: $@" if $@;
    $ctx->stash($MYNAME, $data);

    my $builder = $ctx->stash('builder');
    my $tokens = $ctx->stash('tokens');
    defined (my $out = $builder->build ($ctx, $tokens, $cond))
        or return $ctx->error ($builder->errstr);
    $out;
}

sub _hdlr_tweet_body {
    my ($ctx, $args) = @_;

    my $data = $ctx->stash($MYNAME)
        or return '';
    my $description = $data->{channel}->{item}->[0]->{description};
    Encode::is_utf8 ($description)
        ? Encode::encode ('utf-8', $description)
        : $description;
}

sub _hdlr_tweet_link {
    my ($ctx, $args) = @_;

    my $data = $ctx->stash($MYNAME)
        or return '';
    $data->{channel}->{item}->[0]->{link} || '';
}

sub _hdlr_tweet_date {
    my ($ctx, $args) = @_;

    my $data = $ctx->stash($MYNAME)
        or return '';
    my $time = str2time ($data->{channel}->{item}->[0]->{pubDate});
    local $args->{ts} = MT::Util::epoch2ts ($ctx->stash ('blog'), $time);
    $ctx->build_date ($args);
}

1;
__END__
